/*
功能（Function）: 主界面

作者（Author）:    #AndySun#

创建时间（CreateTime）:  #2017.9.21#
*/
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using XJGame;

public class HomeSceneMainMenuContext : BaseContext
{

	public HomeSceneMainMenuContext () : base (Singleton<UIManager>.Instance.GetUIType ("HomeSceneMainMenuView"))
	{

	}
}

public enum CurrentDisplayInterfaceType
{
	Character,
	Gun,
	Helmet,
}

public class HomeSceneMainMenuView : BaseView
{
	public static HomeSceneMainMenuView Instance;

	private Button setupButton;
	private Button buygoldButton;
	private Button characterButton;
	private Button gunButton;
	private Button helmetButton;
	private Button liveButton;
	private Button dailySignButton;
	private Button startButton;
	private Button removeAdsButton;

	private GameObject playerObj;
	private GameObject gunObj;
	private GameObject hairObj;
	private GameObject liveObj;

	private Image gunImage;
	private Image headImage;
	private Image characterImage;
	private Transform bgObj;

	private GameObject characterObj;
	private GameObject weaponObj;
	private Transform parentTran;
	private Button shopBtn;

	private string gunId;
	private string headId;
	private string characterId;
	[HideInInspector]
	public string currentId;

	private Text liveCountText;

	private Text currentBloodText;
	private Text nextBloodText;
	private Text helmetBloodText;
	//当前角色等级
	private Text currentLevelText;
	private Text expText;
	private Image expImage;

	private bool isFirstCreateGun = true;
	private CurrentDisplayInterfaceType currentDisplayInterfaceType;

	void OnEnable ()
	{
		Instance = this;
		Singleton<XJEventManager>.Instance.AddListener (XJEventType.EQUIP_CHARACTER, OnUpgradeCharacterEnd);
		Singleton<XJEventManager>.Instance.AddListener (XJEventType.EQUIP_GUN, OnChangeGunEnd);
		Singleton<XJEventManager>.Instance.AddListener (XJEventType.EQUIP_HEAD, OnUpgradeHeadEnd);
		Singleton<XJEventManager>.Instance.AddListener (XJEventType.DAILYGIFT_END, OnDaliyGiftEnd);
		Singleton<XJEventManager>.Instance.AddListener (XJEventType.BuyLives_End, RefreshLives);
		AudioManager.Instance.PlayRandomMusic (MusicType.bg0, 5);
	}

	void OnDisable ()
	{
		Singleton<XJEventManager>.Instance.RemoveListener (XJEventType.EQUIP_CHARACTER, OnUpgradeCharacterEnd);
		Singleton<XJEventManager>.Instance.RemoveListener (XJEventType.EQUIP_GUN, OnChangeGunEnd);
		Singleton<XJEventManager>.Instance.RemoveListener (XJEventType.EQUIP_HEAD, OnUpgradeHeadEnd);
		Singleton<XJEventManager>.Instance.RemoveListener (XJEventType.DAILYGIFT_END, OnDaliyGiftEnd);
		Singleton<XJEventManager>.Instance.RemoveListener (XJEventType.BuyLives_End, RefreshLives);
		AudioManager.Instance.StopMusic ();
	}

	void OnDestroy ()
	{
		Instance = null;

	}

	public override void InitOnce ()
	{
		base.InitOnce ();
		setupButton = transform.Find ("setup_button").GetComponent<Button> ();
		buygoldButton = transform.Find ("gold/buycoin").GetComponent<Button> ();
		characterButton = transform.Find ("bg/cosplaybutton/character").GetComponent<Button> ();
		gunButton = transform.Find ("bg/cosplaybutton/gun").GetComponent<Button> ();
		helmetButton = transform.Find ("bg/cosplaybutton/helmet").GetComponent<Button> ();
		liveButton = transform.Find ("bg/cosplaybutton/live").GetComponent<Button> ();
		dailySignButton = transform.Find ("bg/Dailysign").GetComponent<Button> ();
		startButton = transform.Find ("bg/strat").GetComponent<Button> ();
		removeAdsButton = transform.Find ("removeAds").GetComponent<Button> ();
		playerObj = transform.Find ("bg/detal_player").gameObject;
		gunObj = transform.Find ("bg/detal_gun").gameObject;
		hairObj = transform.Find ("bg/detal_helmet").gameObject;
		liveObj = transform.Find ("bg/detal_live").gameObject;
		gunImage = transform.Find ("bg/cosplaybutton/gun/Image").GetComponent<Image> ();
		headImage = transform.Find ("bg/cosplaybutton/helmet/Image").GetComponent<Image> ();
		characterImage = transform.Find ("bg/cosplaybutton/character/Image").GetComponent<Image> ();
		bgObj = transform.Find ("bg");
		shopBtn = transform.Find ("bg/shop").GetComponent<Button> ();
		liveCountText = transform.Find ("bg/cosplaybutton/live/Text").GetComponent<Text> ();
		currentBloodText = transform.Find ("bg/detal/currentHp/value").GetComponent<Text> ();
		nextBloodText = transform.Find ("bg/detal/nextHp/value").GetComponent<Text> ();
		helmetBloodText = transform.Find ("bg/detal/helmetHp/value").GetComponent<Text> ();
		currentLevelText = transform.Find ("bg/detal/lv/lv").GetComponent<Text> ();
		expText = transform.Find ("bg/detal/lv/value").GetComponent<Text> ();
		expImage = transform.Find ("bg/detal/lv/bardown/barup").GetComponent<Image> ();

		if (PlayerPrefs.HasKey ("dailygiftend")) {
			if (PlayerPrefs.GetInt ("dailygiftend") == 1) {
				dailySignButton.gameObject.SetActive (false);
			}
		}
			
		OnButtonClick ();

	}

	public override void OnEnter (BaseContext context)
	{

		base.OnEnter (context);
		PluginManager.Instance.RequestInterstitial (AdsPosition.GameMiddle);
		switch (HomeSceneManager.enterHomeSceneMode) {
		case EnterHomeSceneMode.ChooseModeView:
			//默认先进入主界面的角色选择窗口，再push出选择界面
			FirstCharacter ();
			Singleton<ContextManager>.Instance.Push (new GameModeContext ());
			break;
		case EnterHomeSceneMode.HomeView_character:
			FirstCharacter ();
			break;
		case EnterHomeSceneMode.HomeView_gun:
			Gun ();
			break;
		case EnterHomeSceneMode.HomeView_helmet:
			Hair ();
			break;
		}

		//如果评价状态为提示，则生成评价弹窗
		if (XJGameFramework.Instance.AllowRateusPopup) {
			Singleton<ContextManager>.Instance.Push (new RateUsContext ());
		}

		Init ();
	}

	public override void OnExit (BaseContext context)
	{
		
		base.OnExit (context);
	}

	public override void OnPause (BaseContext context)
	{
		base.OnPause (context);
	}

	public override void OnResume (BaseContext context)
	{
		base.OnResume (context);
	}

	private void Init ()
	{
		RefreshDetal ();
		GenerateCharacter ();

	}

	/// <summary>
	/// 生成角色
	/// </summary>
	private void GenerateCharacter ()
	{
		//新生成一个角色
		if (characterObj == null) {
			GetCharacter ();
		} else {
			Destroy (characterObj);
			GetCharacter ();
		}
	}

	private void GetCharacter ()
	{
		GameObject prefab = Resources.Load ("Prefab/player/" + GameData.Instance.characterDataDictionary [characterId].resourceName) as GameObject;
		characterObj = Instantiate (prefab) as GameObject;
		characterObj.GetComponent<PlayerController> ().ChangeCharacter (headId);
		Utility.SetLayerId (characterObj.transform, LayerMask.NameToLayer ("body_player"));
		characterObj.transform.localScale = Vector3.one * 0.01f;
		if (XJGame.Utility.DeviceAspect <= 1.6f) {
			characterObj.transform.localPosition = new Vector3 (-4.7f, -1.7f, 0);
		} else {
			characterObj.transform.localPosition = new Vector3 (-5.7f, -1.7f, 0);
		}
		parentTran = characterObj.transform.Find ("skeleton/hip/body/arm_front/bigarm/forearm/hand/gun").transform;
		isFirstCreateGun = true;
		GenerateGun ();
	}

	/// <summary>
	/// 生成枪
	/// </summary>
	private void GenerateGun ()
	{
		//新生成一个角色
		if (weaponObj == null) {
			GetGun ();
		} else {
			Destroy (weaponObj);
			GetGun ();
		}
	}

	private void GetGun ()
	{
		GameObject prefab = Resources.Load ("Prefab/gun/" + GameData.Instance.gunDataDictionary [gunId].iconName) as GameObject;
		weaponObj = Instantiate (prefab) as GameObject;
		weaponObj.transform.SetParent (parentTran);
		weaponObj.transform.localPosition = Vector3.zero;
		weaponObj.transform.localScale = Vector3.one;
		if (characterObj.GetComponent<PlayerController> ().SkType == SkeletonType.Man) {
			weaponObj.transform.localRotation = Quaternion.Euler (Vector3.zero);
		} else {
			weaponObj.transform.localRotation = Quaternion.Euler (new Vector3 (0, 0, 25f));
		}

		Utility.SetLayerId (weaponObj.transform, LayerMask.NameToLayer ("body_player"));
		if (isFirstCreateGun) {
			weaponObj.transform.Find ("gun").GetComponent<SpriteRenderer> ().sortingOrder = -4;
			isFirstCreateGun = false;
		} else {
			weaponObj.transform.Find ("gun").GetComponent<SpriteRenderer> ().sortingOrder = -8;
		}
	}

	void Update ()
	{
		if (Input.GetKeyDown (KeyCode.Escape)) {
			//显示退出界面
			AudioManager.Instance.PlayTone (ToneType.ButtonClick);
			Singleton<ContextManager>.Instance.Push (new QuitGameContext ());
		}
	}
		
	//点击按钮
	private void OnButtonClick ()
	{
		setupButton.onClick.AddListener (Setup);
		buygoldButton.onClick.AddListener (BuyCoin);
		characterButton.onClick.AddListener (Character);
		gunButton.onClick.AddListener (Gun);
		helmetButton.onClick.AddListener (Hair);
		liveButton.onClick.AddListener (Live);
		dailySignButton.onClick.AddListener (DailySign);
		startButton.onClick.AddListener (StartMode);
		shopBtn.onClick.AddListener (BuyCoin);
		removeAdsButton.onClick.AddListener (RemoveAds);

	}

	/// <summary>
	/// 显示购买金币界面
	/// </summary>
	private void BuyCoin ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		Singleton<ContextManager>.Instance.Push (new BuyCoinContext ());

	}

	/// <summary>
	/// 显示移除广告界面
	/// </summary>
	private void RemoveAds(){
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		Singleton<ContextManager>.Instance.Push (new RemoveAdsContext ());
	}

	/// <summary>
	/// 显示设置界面
	/// </summary>
	private void Setup ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		Singleton<ContextManager>.Instance.Push (new SettingContext ());
	}

	/// <summary>
	/// 显示每日签到界面
	/// </summary>
	private void DailySign ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		Singleton<ContextManager>.Instance.Push (new SignInContext ());
	}

	/// <summary>
	/// 显示选模式界面
	/// </summary>
	private	void StartMode ()
	{
		
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		Singleton<ContextManager>.Instance.Push (new GameModeContext ());
		switch (currentDisplayInterfaceType) {
		case CurrentDisplayInterfaceType.Character:
			ChosePlayerUIManager.Instance.Init ();
			ChosePlayerUIManager.Instance.GetIndexByCharacterId ();
			break;
		case CurrentDisplayInterfaceType.Gun:
			ChoseGunUIManager.Instance.Init ();
			ChoseGunUIManager.Instance.GetIndexByGunId ();
			break;
		case CurrentDisplayInterfaceType.Helmet:
			ChoseHelmetUIManager.Instance.Init ();
			ChoseHelmetUIManager.Instance.GetIndexByHelmetId ();
			break;

		}

	}

	/// <summary>
	/// 显示角色信息
	/// </summary>
	private void FirstCharacter ()
	{
		playerObj.SetActive (true);
		gunObj.SetActive (false);
		hairObj.SetActive (false);
		liveObj.SetActive (false);
		ChosePlayerUIManager.Instance.Init ();
		ChosePlayerUIManager.Instance.GetIndexByCharacterId ();
		currentDisplayInterfaceType = CurrentDisplayInterfaceType.Character;

	}

	/// <summary>
	/// 显示角色信息
	/// </summary>
	private void Character ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		playerObj.SetActive (true);
		gunObj.SetActive (false);
		hairObj.SetActive (false);
		liveObj.SetActive (false);
		ChosePlayerUIManager.Instance.Init ();
		ChosePlayerUIManager.Instance.GetIndexByCharacterId ();
		currentDisplayInterfaceType = CurrentDisplayInterfaceType.Character;

	}

	/// <summary>
	/// 显示枪信息
	/// </summary>
	private void Gun ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		playerObj.SetActive (false);
		gunObj.SetActive (true);
		hairObj.SetActive (false);
		liveObj.SetActive (false);
		ChoseGunUIManager.Instance.Init ();
		ChoseGunUIManager.Instance.GetIndexByGunId ();
		currentDisplayInterfaceType = CurrentDisplayInterfaceType.Gun;

	}

	/// <summary>
	/// 显示头盔信息
	/// </summary>
	private void Hair ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		playerObj.SetActive (false);
		gunObj.SetActive (false);
		hairObj.SetActive (true);
		liveObj.SetActive (false);
		ChoseHelmetUIManager.Instance.Init ();
		ChoseHelmetUIManager.Instance.GetIndexByHelmetId ();
		currentDisplayInterfaceType = CurrentDisplayInterfaceType.Helmet;
	}

	/// <summary>
	/// 显示复活丹信息
	/// </summary>
	private void Live ()
	{
		AudioManager.Instance.PlayTone (ToneType.ButtonClick);
		playerObj.SetActive (false);
		gunObj.SetActive (false);
		hairObj.SetActive (false);
		liveObj.SetActive (true);
	}

	/// <summary>
	/// 刷新
	/// </summary>
	private void RefreshLives (object sender, System.EventArgs args)
	{
		liveCountText.text = GameData.Instance.profileData.Lives + "";
	}


	/// <summary>
	/// 刷新详情
	/// </summary>
	private void RefreshDetal ()
	{
		gunId = GameData.Instance.profileData.CurrentGunId;
		headId = GameData.Instance.profileData.CurrentHelmetId;
		characterId = GameData.Instance.profileData.CurrentCharacterId;
		Dictionary<string,CharacterData> characterDataDictionary = GameData.Instance.characterDataDictionary;
		Dictionary<string,HelmetData> helmetDataDictionary = GameData.Instance.helmetDataDictionary;

		//按钮选择显示当前对应的ICON
		headImage.sprite = Resources.Load<Sprite> ("Texture/helmet/" + helmetDataDictionary [headId].iconName);
		gunImage.sprite = Resources.Load<Sprite> ("Texture/gun/" + GameData.Instance.gunDataDictionary [gunId].iconName);
		characterImage.sprite = Resources.Load<Sprite> ("Texture/character/" + characterDataDictionary [characterId].iconName);

		//复活丹数量
		liveCountText.text = GameData.Instance.profileData.Lives + "";

		int currentLevel = characterDataDictionary [characterId].ExpToLevel ();
		switch (currentLevel) {
		case 0:
			currentBloodText.text = characterDataDictionary [characterId].blood + "";
			nextBloodText.text = characterDataDictionary [characterId].blood + characterDataDictionary [characterId].levelUpBlood + "";
			helmetBloodText.text = helmetDataDictionary [headId].bloodAdd + "";
			break;
		case 9:
			currentBloodText.text = characterDataDictionary [characterId].blood + currentLevel * characterDataDictionary [characterId].levelUpBlood + "";
			nextBloodText.text = "--";
			helmetBloodText.text = helmetDataDictionary [headId].bloodAdd + "";
			break;
		default:
			currentBloodText.text = characterDataDictionary [characterId].blood + currentLevel * characterDataDictionary [characterId].levelUpBlood + "";
			nextBloodText.text = characterDataDictionary [characterId].blood + (currentLevel + 1) * characterDataDictionary [characterId].levelUpBlood + "";
			helmetBloodText.text = helmetDataDictionary [headId].bloodAdd + "";
			break;
		}
		currentLevelText.text = "LV." + (currentLevel + 1);
		if (currentLevel > 8) {
			currentLevel = 8;
		}

		//当前经验
		int currentExp = characterDataDictionary [characterId].CurrentExp;
		//当前剩余经验
		int currentRemainExp = characterDataDictionary [characterId].GetRemainExp(currentExp);
		//下次升级经验
		int levelUpExp = GameData.Instance.expDataDictionary [characterDataDictionary [characterId].expId].expRewardList [currentLevel];

		expText.text = currentRemainExp + "/" + levelUpExp;
		expImage.fillAmount = (float)currentRemainExp / (float)levelUpExp;

		if (characterDataDictionary [characterId].ExpToLevel () >= 9) {
			expText.text = LTLocalization.GetText("73");
			expImage.fillAmount = 1;
		}

	}

	public void OnChangeGunEnd (object sender, System.EventArgs args)
	{

		RefreshDetal ();
		GenerateGun ();
	}

	public void OnUpgradeHeadEnd (object sender, System.EventArgs args)
	{
		
		RefreshDetal ();
		characterObj.GetComponent<PlayerController> ().ChangeDecorate (headId);
	}

	public void OnUpgradeCharacterEnd (object sender, System.EventArgs args)
	{
		RefreshDetal ();
		GenerateCharacter ();
	}

	public void OnDaliyGiftEnd (object sender, System.EventArgs args)
	{

		dailySignButton.gameObject.SetActive (false);
	}

}

